/*   AM-CLR for ICP-PCI Kernel module free public release
 *
 *   Werner Jaeger        email: e9026603@student.tuwien.ac.at
 *   Wilfried Kubinger    email: wk@flexaut.tuwien.ac.at
 *
 *   Institute of Flexible Automation
 *   University of Technology Vienna
 *   Austria, Europe
 *
 *   amclrReg.h  v1.1  1999/03/02
 */

/* Change History:

   - 1999/05/03 12:47:07 matthias
     ajustments to version 0.5.0 (card8/16 -> u_char/short)

   - Add c1_dump_regs prototype.

   - Add def C1_REG_SIZE.

 */

#ifndef _AMCLRREG_H
#define _AMCLRREG_H

#if defined(linux) && defined(__KERNEL__) || defined(__CONFIG__)

/* AM-CLR Hardware Reference Manual; Imaging Technology Inc.*/
/* Document Number 47-H44003-02; January 1994 */

/* R-O   Read-Only Register */
/* W-O   Write-Only Register */
/* R/W   Read/Write Register */
/* R/W1C Clear on Write One to Register */
/* R/W0C Clear on Write Zero to Register */

/* AM-CLR Register Set (16bit access only), pages 3-1 to 3-24 */
/* ********************************************************** */

/* 
 * Prototypes
 */

/* read handler */

u_char c1_get_rev (u_char);
u_char c1_get_type (u_char);
u_char c1_get_camsel (u_char);
u_char c1_get_svhs (u_char);
u_char c1_get_trgen0 (u_char);
u_char c1_get_trgen1 (u_char);
u_char c1_get_trgpol (u_char);
u_char c1_get_synctyp (u_char);
u_char c1_get_pclkpol (u_char);
u_char c1_get_lowpass (u_char);
u_char c1_get_islvl (u_char);
u_char c1_get_color (u_char);
u_char c1_get_syncsel (u_char);
u_char c1_get_bypass_olut (u_char);

int c1_get_csc (u_char, struct c1_csc_t *);
int c1_get_ilut (u_char, struct c1_ilut_t *);
int c1_get_olut (u_char, struct c1_olut_t *);
int c1_get_ref (u_char, struct c1_ref_t *);

void c1_get_regs (u_char, struct c1_regs_t *);

/* write handler */

int c1_set_camsel (u_char, u_char);
int c1_set_svhs (u_char, u_char);
int c1_set_trgen0 (u_char, u_char);
int c1_set_trgen1 (u_char, u_char);
int c1_set_trgpol (u_char, u_char);
int c1_set_synctyp (u_char, u_char);
int c1_set_pclkpol (u_char, u_char);
int c1_set_strobe1 (u_char, u_char);
int c1_set_strobe2 (u_char, u_char);
int c1_set_lowpass (u_char, u_char);
int c1_set_islvl (u_char, u_char);
int c1_set_color (u_char, u_char);
int c1_set_syncsel (u_char, u_char);
int c1_set_bypass_olut (u_char, u_char);

int c1_set_fpga (u_char, struct c1_fpga_t *);
int c1_set_csc (u_char, struct c1_csc_t *);
int c1_set_ilut (u_char, struct c1_ilut_t *);
int c1_set_olut (u_char, struct c1_olut_t *);
int c1_set_ref (u_char, struct c1_ref_t *);

/* init handler */

int c1_init_regs (u_char);

/* misc handler */

int c1_linearize_olut (u_char);
int c1_linearize_ilut (u_char);
void c1_dump_regs (u_char);

/* end of prototypes */

#define C1_REG_SIZE             32  /* AM-CLR Regs 0x00 - 0x1F */  

/*
 * AM-CLR Register Definitions 
 */

/*-------------------------------------------------------------------------*/
/*      u_char   modid           Module-ID Register                         */
/*-------------------------------------------------------------------------*/

#define C1_MODID_OFFSET         0x00

/*      u_char   rev:3          Revision [R-O] */

#define C1_REV_MASK             0x07
#define C1_REV_SHIFT            0

/*      u_char   type:5         Board Type [R-O] */

#define C1_TYPE_MASK            0xF8
#define C1_TYPE_SHIFT           3


/*-------------------------------------------------------------------------*/
/*      u_char   fpga           FPGA Access Register                       */
/*-------------------------------------------------------------------------*/

#define C1_FPGA_OFFSET          0x01


/*-------------------------------------------------------------------------*/
/*      u_char   0x02           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   camcon         Camera Control Register                    */
/*-------------------------------------------------------------------------*/

#define C1_CAMCON_OFFSET        0x03

#define C1_CAMCON_RGB           0x00    /* Camera 0 selected */           
				        /* SuperVHS disabled  */           
				        /* external Trigger 0 disabled */ 
				        /* external Trigger 1 disabled */ 
				        /* No Invert Trigger Polatity */  

#define C1_CAMCON_NTSC          0x02    /* Camera 2 selected */           
				        /* SuperVHS disabled  */           
				        /* external Trigger 0 disabled */ 
				        /* external Trigger 1 disabled */ 
				        /* No Invert Trigger Polatity */  

#define C1_CAMCON_PAL           0x02    /* Camera 2 selected */           
				        /* SuperVHS disabled  */           
				        /* external Trigger 0 disabled */ 
				        /* external Trigger 1 disabled */ 
				        /* No Invert Trigger Polatity */  

#define C1_CAMCON_SVHS          0x06    /* Camera 2 selected */
					/* SuperVHS enabled  */
					/* external Trigger 0 disabled */
					/* external Trigger 1 disabled */
					/* No Invert Trigger Polatity */  

#define C1_CAMCON_MONO          0x00    /* Camera 0 selected */           
				        /* SuperVHS disabled  */           
				        /* external Trigger 0 disabled */ 
				        /* external Trigger 1 disabled */ 
				        /* No Invert Trigger Polatity */  

/*      u_char   camsel:2       Camera Select [R/W] */

#define C1_CAMSEL_MASK          0x03
#define C1_CAMSEL_SHIFT         0
#define C1_CAMSEL_CAM0          0
#define C1_CAMSEL_CAM1          1
#define C1_CAMSEL_CAM2          2
#define C1_CAMSEL_CAM3          3

/*      u_char   svhs:1         SuperVHS Select [R/W] */

#define C1_SVHS_MASK            0x04
#define C1_SVHS_SHIFT           2
#define C1_SVHS_DISABLED        0
#define C1_SVHS_ENABLED         1

/*      u_char   trgen0:1       external Trigger 0 [R/W] */

#define C1_TRGEN0_MASK          0x08
#define C1_TRGEN0_SHIFT         3
#define C1_TRGEN0_DISABLED      0
#define C1_TRGEN0_ENABLED       1

/*      u_char   trgen1:1       external Trigger 1 [R/W] */

#define C1_TRGEN1_MASK          0x10
#define C1_TRGEN1_SHIFT         4
#define C1_TRGEN1_DISABLED      0
#define C1_TRGEN1_ENABLED       1

/*      u_char   trgpol:1       Trigger Polarity [R/W] */

#define C1_TRGPOL_MASK          0x20
#define C1_TRGPOL_SHIFT         5
#define C1_TRGPOL_NOINVERT      0
#define C1_TRGPOL_INVERT        1

/*      u_char   res:2          reserved - reset to 0 */


/*-------------------------------------------------------------------------*/
/*      u_char   modcon         Module Control Register                    */
/*-------------------------------------------------------------------------*/

#define C1_MODCON_OFFSET        0x04

#define C1_MODCON_RGB           0x0C    /* Composite TTL Sync Input */   
				        /* No Invert Pixel Clock */   
				        /* Strobe 0 Output Low */     
				        /* Strobe 1 Output Low */     
                		        /* Low-Pass Filter disabled */

#define C1_MODCON_NTSC          0x0D    /* Composite Video Input */    
				        /* No Invert Pixel Clock */   
				        /* Strobe 0 Output Low */     
				        /* Strobe 1 Output Low */     
				        /* Low-Pass Filter disabled */

#define C1_MODCON_PAL           0x0D    /* Composite Video Input */   
				        /* No Invert Pixel Clock */   
				        /* Strobe 0 Output Low */     
				        /* Strobe 1 Output Low */     
				        /* Low-Pass Filter disabled */

#define C1_MODCON_SVHS          0x0D    /* Composite Video Input */   
				        /* No Invert Pixel Clock */   
				        /* Strobe 0 Output Low */     
				        /* Strobe 1 Output Low */     
				        /* Low-Pass Filter disabled */

#define C1_MODCON_MONO          0x0D    /* Composite Video Input */   
                		        /* No Invert Pixel Clock */   
				        /* Strobe 0 Output Low */     
                                        /* Strobe 1 Output Low */     
                                        /* Low-Pass Filter disabled */

/*      u_char   synctyp:4              Sync Type [R/W] */
				        
#define C1_SYNCTYP_MASK         0x0F
#define C1_SYNCTYP_SHIFT        0
#define C1_SYNCTYP_CV_IN        0x0D    /* composite video input */
#define C1_SYNCTYP_CS_IN        0x0C    /* composite sync input  */
#define C1_SYNCTYP_SS_IN        0x0E    /* seperate sync input   */
#define C1_SYNCTYP_CS_OUT       0x08    /* composite sync output */
#define C1_SYNCTYP_SS_OUT       0x0A    /* seperate sync output  */
#define C1_SYNCTYP_SS_OUT_PI    0x03    /* seperate sync out, PCLK in   */
#define C1_SYNCTYP_CS_OUT_PI    0x01    /* composite sync out, PCLK in  */
#define C1_SYNCTYP_SS_OUT_PO    0x0B    /* seperate sync out, PCLK out  */
#define C1_SYNCTYP_CS_OUT_PO    0x09    /* composite sync out, PCLK out */
#define C1_SYNCTYP_SS_IN_PI     0x02    /* seperate sync in, PCLK in    */

/*      u_char   pclkpol:1      Invert Pixel Clock Polarity [R/W] */

#define C1_PCLKPOL_MASK         0x10
#define C1_PCLKPOL_SHIFT        4
#define C1_PCLKPOL_NOINVERT     0
#define C1_PCLKPOL_INVERT       1

/*      u_char   strobe1:1      Strobe 1 Output [W-O] */

#define C1_STROBE1_MASK         0x20
#define C1_STROBE1_SHIFT        5
#define C1_STROBE1_LOW          0
#define C1_STROBE1_HIGH         1

/*      u_char   strobe2:1      Strobe 2 Output [W-O] */

#define C1_STROBE2_MASK         0x40
#define C1_STROBE2_SHIFT        6
#define C1_STROBE2_LOW          0
#define C1_STROBE2_HIGH         1

/*      u_char   lowpass:1      Low Pass Filter [R/W] */

#define C1_LOWPASS_MASK         0x80
#define C1_LOWPASS_SHIFT        7
#define C1_LOWPASS_DISABLED     0
#define C1_LOWPASS_ENABLED      1


/*-------------------------------------------------------------------------*/
/*      u_char   0x05           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   0x06           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   0x07           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   0x08           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   0x09           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   0x0A           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   0x0B           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   cscadrlo       CSC Low-Address Register  [R/W]            */
/*-------------------------------------------------------------------------*/

#define C1_CSCADRLO_OFFSET      0x0C

#define C1_ILUTA_LO             0x00
#define C1_ILUTB_LO             0x00
#define C1_ILUTC_LO             0x00
#define C1_MATRIX_LO            0x00
#define C1_CMD0_LO              0x12
#define C1_CMD1_LO              0x13


/*-------------------------------------------------------------------------*/
/*      u_char   cscadrhi       CSC High-Address Register                  */
/*-------------------------------------------------------------------------*/

#define C1_CSCADRHI_OFFSET      0x0D

#define C1_ILUTA_HI             0x00
#define C1_ILUTB_HI             0x01
#define C1_ILUTC_HI             0x02
#define C1_MATRIX_HI            0x03
#define C1_CMD0_HI              0x03
#define C1_CMD1_HI              0x03

/*      u_char   cscadrhi:4     CSC Address Bit 9 - 12 [R/W] */

#define C1_CSCADRHI_MASK        0x0F
#define C1_CSCADRHI_SHIFT       0

/*      u_char   res:4          reserved - reset to 0 */


/*-------------------------------------------------------------------------*/
/*      u_char   cscdat         CSC Data Register  [R/W]  (Auto-INC Adr)   */
/*-------------------------------------------------------------------------*/

#define C1_CSCDAT_OFFSET        0x0E

/*      Overflow/Underflow Flag of CMD0 register */

#define C1_CSC_OVERFLOW_MASK    0x01
#define C1_CSC_OVERFLOW_SHIFT   0

/*      Error Flag of CMD0 register */

#define C1_CSC_ERROR_MASK       0x02
#define C1_CSC_ERROR_SHIFT      1

/*      matrix informat values for CMD0 register */

#define C1_CSC_IN_MASK          0xE0
#define C1_CSC_UMAGIN           0x00       /* unsigned magnitude */
#define C1_CSC_SMAGIN           0x20       /* analog sign magnitude */
#define C1_CSC_2COMPIN          0x80       /* 2's compliment signed */
#define C1_CSC_PSEUDOIN         0xE0       /* pseudo-color mode */

/*      matrix outformat values for CMD0 register */

#define C1_CSC_OUT_MASK         0x14
#define C1_CSC_UMAGOUT          0x00       /* unsigned magnitude */
#define C1_CSC_SMAGOUT          0x04       /* analog sign magnitude */
#define C1_CSC_2COMPOUT         0x10       /* 2's compliment signed */

/*      matrix shift values for CMD1 register */

#define C1_CSC_SHIFT_MASK       0xE0
#define C1_CSC_SHIFT1           0x00       /* scale by 1 */
#define C1_CSC_SHIFT2           0x20       /* scale by 2 */
#define C1_CSC_SHIFT4           0x40       /* scale by 4 */
#define C1_CSC_SHIFT8           0x60       /* scale by 8 */
#define C1_CSC_SHIFT16          0x80       /* scale by 16 */
#define C1_CSC_BYPASS           0xE0       /* bypass entire CSC */
     

/*-------------------------------------------------------------------------*/
/*      u_char   0x0F           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   adccon         ADC Control Register                       */
/*-------------------------------------------------------------------------*/
 
#define C1_ADCCON_OFFSET        0x10

#define C1_ADCCON_RGB           0x20   /* 125mV Sync Level Threshold */ 
				       /* 24 Bit True Color */          
				       /* Sync from Green Video Input */

#define C1_ADCCON_NTSC          0x20   /* 125mV Sync Level Threshold */ 
				       /* 24 Bit True Color */          
				       /* Sync from Green Video Input */

#define C1_ADCCON_PAL           0x20   /* 125mV Sync Level Threshold */ 
				       /* 24 Bit True Color */          
				       /* Sync from Green Video Input */

#define C1_ADCCON_SVHS          0x20   /* 125mV Sync Level Threshold */
				       /* 24 Bit True Color */
				       /* Sync from Green Video Input */

#define C1_ADCCON_MONO          0x2C   /* 125mV Sync Level Threshold */ 
				       /* 8 Bit Pseudo Color */          
				       /* Sync from Green Video Input */

/*      u_char   islvl:1        Sync Strip Level Threshold [R/W] */ 

#define C1_ISLVL_MASK           0x01
#define C1_ISLVL_SHIFT          0
#define C1_ISLVL_125MV          0
#define C1_ISLVL_50MV           1

/*      u_char   rev:1          reserved - reset to 0 */

/*      u_char   color:2        Color Mode Select [R/W] */

#define C1_COLOR_MASK           0x0C
#define C1_COLOR_SHIFT          2
#define C1_COLOR_24TRUE         0
#define C1_COLOR_15TRUE         1
#define C1_COLOR_8TRUE          2
#define C1_COLOR_8PSEUDO        3

/*      u_char   syncsel:3      Sync Strip Source Select [R/W] */

#define C1_SYNCSEL_MASK         0x70
#define C1_SYNCSEL_SHIFT        4
#define C1_SYNCSEL_RED          0
#define C1_SYNCSEL_GREEN        2
#define C1_SYNCSEL_BLUE         4
#define C1_SYNCSEL_EXT0         6
#define C1_SYNCSEL_EXT1         7

/*      u_char   vsel:1         must be reset to 0 */


/*-------------------------------------------------------------------------*/
/*      u_char   rnref          Red negative Reference Register            */
/*-------------------------------------------------------------------------*/
 
#define C1_RNREF_OFFSET         0x11

#define C1_RNREF_MONO           0x20
#define C1_RNREF_RGB            0x20
#define C1_RNREF_NTSC           0x4C
#define C1_RNREF_PAL            0x4C
#define C1_RNREF_SVHS           0x4C

/*      u_char   res:2          reserved - reset to 0 */

/*      u_char   rnref:6        Red Neg Ref Voltage [R/W] */

#define C1_RNREF_MASK           0xFC
#define C1_RNREF_SHIFT          2


/*-------------------------------------------------------------------------*/
/*      u_char   rpref          Red positive Reference Register            */
/*-------------------------------------------------------------------------*/
 
#define C1_RPREF_OFFSET         0x12

#define C1_RPREF_MONO           0x74
#define C1_RPREF_RGB            0x74
#define C1_RPREF_NTSC           0xF0
#define C1_RPREF_PAL            0xF0
#define C1_RPREF_SVHS           0xF0

/*      u_char   res:2          reserved - reset to 0 */

/*      u_char   rpref:6        Red Pos Ref Voltage [R/W] */

#define C1_RPREF_MASK           0xFC
#define C1_RPREF_SHIFT          2


/*-------------------------------------------------------------------------*/
/*      u_char   gnref          Green negative Reference Register          */
/*-------------------------------------------------------------------------*/
 
#define C1_GNREF_OFFSET         0x13

#define C1_GNREF_MONO           0x20
#define C1_GNREF_RGB            0x20
#define C1_GNREF_NTSC           0x08
#define C1_GNREF_PAL            0x08
#define C1_GNREF_SVHS           0x20

/*      u_char   res:2          reserved - reset to 0 */

/*      u_char   gnref:6        Green Neg Ref Voltage [R/W] */

#define C1_GNREF_MASK           0xFC
#define C1_GNREF_SHIFT          2


/*-------------------------------------------------------------------------*/
/*      u_char   gpref          Green positive Reference Register          */
/*-------------------------------------------------------------------------*/
 
#define C1_GPREF_OFFSET         0x14

#define C1_GPREF_MONO           0x74
#define C1_GPREF_RGB            0x74
#define C1_GPREF_NTSC           0x48
#define C1_GPREF_PAL            0x48
#define C1_GPREF_SVHS           0x70

/*      u_char   res:2          reserved - reset to 0 */

/*      u_char   gpref:6        Green Pos Ref Voltage [R/W] */

#define C1_GPREF_MASK           0xFC
#define C1_GPREF_SHIFT          2


/*-------------------------------------------------------------------------*/
/*      u_char   bnref          Blue negative Reference Register           */
/*-------------------------------------------------------------------------*/
 
#define C1_BNREF_OFFSET         0x15

#define C1_BNREF_MONO           0x20
#define C1_BNREF_RGB            0x20
#define C1_BNREF_NTSC           0x4C
#define C1_BNREF_PAL            0x4C
#define C1_BNREF_SVHS           0x4C

/*      u_char   res:2          reserved - reset to 0 */

/*      u_char   bnref:6        Blue Neg Ref Voltage [R/W] */

#define C1_BNREF_MASK           0xFC
#define C1_BNREF_SHIFT          2


/*-------------------------------------------------------------------------*/
/*      u_char   bpref          Blue positive Reference Register           */
/*-------------------------------------------------------------------------*/
 
#define C1_BPREF_OFFSET         0x16

#define C1_BPREF_MONO           0x74
#define C1_BPREF_RGB            0x74
#define C1_BPREF_NTSC           0xF0
#define C1_BPREF_PAL            0xF0
#define C1_BPREF_SVHS           0xF0

/*      u_char   res:2          reserved - reset to 0 */

/*      u_char   bpref:6        Blue Pos Ref Voltage [R/W] */

#define C1_BPREF_MASK           0xFC
#define C1_BPREF_SHIFT          2


/*-------------------------------------------------------------------------*/
/*      u_char   0x17           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   alutadr        Output LUT A Address Register  [R/W]       */
/*-------------------------------------------------------------------------*/

#define C1_ALUTADR_OFFSET       0x18


/*-------------------------------------------------------------------------*/
/*      u_char   blutadr        Output LUT B Address Register  [R/W]       */
/*-------------------------------------------------------------------------*/

#define C1_BLUTADR_OFFSET       0x19

#define C1_BLUTADR_INIT         0x00    /* Don't bypass Output LUTs */

/*      u_char   blutadr:3       B LUT Address Bit 9 - 10 [R/W] */

#define C1_BLUTADR_MASK         0x07
#define C1_BLUTADR_SHIFT        0

/*      u_char   bypass:1        Bypass Output LUTs [R/W] */

#define C1_BYPASS_OLUT_MASK     0x08
#define C1_BYPASS_OLUT_SHIFT    3
#define C1_BYPASS_OLUT_DISABLED 0
#define C1_BYPASS_OLUT_ENABLED  1

/*      u_char   res:4           reserved - reset to 0 */


/*-------------------------------------------------------------------------*/
/*      u_char   clutadr        Output LUT C Address Register  [R/W]       */
/*-------------------------------------------------------------------------*/

#define C1_CLUTADR_OFFSET       0x1A


/*-------------------------------------------------------------------------*/
/*      u_char   alutdat        Output LUT A Data Register  [R/W]          */
/*-------------------------------------------------------------------------*/

#define C1_ALUTDAT_OFFSET       0x1B


/*-------------------------------------------------------------------------*/
/*      u_char   blutdat        Output LUT B Data Register  [R/W]          */
/*-------------------------------------------------------------------------*/

#define C1_BLUTDAT_OFFSET       0x1C


/*-------------------------------------------------------------------------*/
/*      u_char   0x1D           Output LUT C Data Register  [R/W]          */
/*-------------------------------------------------------------------------*/

#define C1_CLUTDAT_OFFSET       0x1D


/*-------------------------------------------------------------------------*/
/*      u_char   0x1E           reserved                                   */
/*-------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------*/
/*      u_char   0x1F           reserved                                   */
/*-------------------------------------------------------------------------*/


#endif /* linux && __KERNEL__ || __CONFIG__ */

#endif /* _AMCLRREG_H */

