/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 *
 * $Id: icpciExt.h,v 5.1.2.3 1999/07/21 08:39:38 matthias Exp $
 *
 */

#ifndef _ICPCIEXT_H
#define _ICPCIEXT_H

/*
 * Includes:
 */

/* #undef linux */
#undef EXT_OS_DEFINED

#ifdef linux
#  define EXT_OS_DEFINED
#  ifdef __KERNEL__
#    include <linux/time.h>	/* struct timeval/timespec */
#    include <linux/ioctl.h>	/* _IOR/W ... */
#  else
#    include <sys/types.h>	/* basic data types */
#    include <sys/time.h>	/* struct timeval/timespec */
#    include <sys/ioctl.h>	/* _IOR/W ... */
#  endif
#endif

/*
 * Defines:
 */

/* function return values */
#define retval	int
#define OK	0
#define ERROR	-1

/* hardware bit values */
#define bit	u_char
#define LOW	0
#define HIGH	1

/* boolean logic values */
#ifndef FALSE
#  define FALSE	0
#endif
#ifndef TRUE
#  define TRUE	1
#endif

/* pointer init/error value */
#ifndef NULL
#  define NULL	(void*)0
#endif

/* if nothing is defined, fallback */
#ifndef u_char
#  define u_char	unsigned char
#endif
#ifndef u_short
#  define u_short	unsigned short
#endif
#ifndef u_long
#  define u_long	unsigned long
#endif
#ifndef u_int
#  define u_int		unsigned int
#endif

/* error handling (kernel and user space) */
#ifdef DEBUG_ICPCI
#  if defined(linux) && defined(__KERNEL__)
#    define ICP_PRINTF(fmt,args...) icp_printf (fmt, ## args)
#  else
#    define ICP_PRINTF(fmt,args...) fprintf (stderr,  fmt, ## args); \
				    fflush (stderr);
#  endif
#else
#  define ICP_PRINTF(fmt,args...)
#endif

/* global structure limits */
#define ICP_BOARDS_MAX	4	/* max. number of boards to plug in */
#define ICP_CAMERAS_MAX	4	/* maximal number of cameras per module */
#define ICP_BUFFERS_MAX	48	/* maximal number of memory targets */
#define ICP_MODULE_MAX	32	/* module number space */

/* shortkeys for acquisition modules */
#define ICP_AMNO	0	/* No Acquisition Module Installed */ 
#define ICP_AMFA	1	/* Fast Acquisition Module */
#define ICP_AMC1	2	/* Color Acquisition Module */
#define ICP_AMVS	3	/* Variable Scan Module */
#define ICP_AMDG	4	/* Digital Module */
#define ICP_AMMTD	6	/* Multiple Tapped Module */
#define ICP_AMCMP	7	/* Standard Composite Sync Module */
#define ICP_AMRGB	8	/* Standard RGB Module */
#define ICP_AMPV	16	/* PCVision */ 

/* maximum number of cameras per acq. module */
#define VS_CAMERAS_MAX	4
#define DG_CAMERAS_MAX	1
#define PV_CAMERAS_MAX	4

/* limits for frames per second values */
#define	ICP_FPS_MIN	0.01	/* we support frames per second from */
#define	ICP_FPS_MAX	100.0	/* up to ... */

#define ICP_INIT_FPS	25.0	/* 50 Hz CCIR (interlaced!) */

/* limits for the external sync delay time (AM-DIG) */
#define ICP_DLY_BASE	70	/* time base in nanoseconds (1 / 14.32 MHz) */
#define ICP_DLY_MIN	((ICP_DLY_BASE << 1) * 3)	/* min prescaler and
							   duration counter */
#define ICP_DLY_MAX	((ICP_DLY_BASE << 11) * 4098)	/* max prescaler and
							   duration counter */

/* timeout defines */
#define ICP_WAIT_IOP	20 * HZ	/* inport data waiting time (20s) */
#define ICP_WAIT_TRIG   20 * HZ /* ext/sw trigger waiting time (20s) */

/* maximal number of frames to snap (hardware limit) */
#define ICP_SNAPS_MAX	8

/*
 * Typedefs:
 */

/* AM specific data types */
/* lookup tables */
struct vs_lut_t
{
  u_char lutdata[256];
};

union dg_lut16_t
{
  u_short lut8data[256];
  u_short lut10data[1024];
  u_short lut12data[4096];
  u_short lut16data[65536];
};

struct dg_lut8_t
{
  u_char lutdata[256];
};

#define pv_lut_t vs_lut_t

/* camera structures */
struct vs_cam_t
{
  u_char tsel;
  u_char clksrc;
  u_char lpfsel;
  u_char iscale;
  u_char smode;
  u_char tsrc;
  u_char lenpol;
  u_char fldpol;
  u_char fenpol;
  u_char pclkpol;
  u_char eclken;
  u_char fldsten;
  u_char vsten;
  u_char trgen;
  u_char trgsrc;
  u_char fmode;
  u_char tmode;
  u_char strbdly;
  u_short hoff;
  u_short hsz;
  u_short voff;
  u_short vsz;
  u_short froff;
  u_char cppos;
  u_char cpen;
  u_char cpsrc;
  u_char strbsel;
  u_char strbpol;
  u_char cpst;
  u_short nref;
  u_short pref;
};

struct dg_cam_t {
  u_char fsel;
  u_char clkdiv;
  u_char xillen;
  u_char mc;
  u_char pclkpol;
  u_char lenpol;
  u_char fenpol;
  u_char fldpol;
  u_char smode;
  u_char ilmode;
  u_char lensm;
  u_char nolmiss;
  u_char lvar;
  u_char exclk;
  u_char extmd;
  u_char exsen;
  u_char expol;
  u_char prien;
  u_char prip;
  u_short extime;
  u_char psize;
  u_char ilpixel;
  u_char muxa;
  u_char muxb;
  u_char muxc;
  u_short hoff;
  u_short hact;
  u_short voff;
  u_short vact;
  u_char trigen;
  u_char trigsrc;
};

#include "ampcvReg.h"	/* we need this for the PCVision camera structure
			   definition - AM-VS and friends are handled in an
			   other way */

struct pv_reg_t {
  pv_modid_t modid;
  u_short reserved_0;
  u_short reserved_1;
  u_short reserved_2;
  u_short lutaddr;
  u_short lutdata;
  u_short adcdata;
  u_short reserved_3;
  u_short htotal;
  u_short hsyncpw;
  u_short hequipw;
  u_short hserrpw;
  u_short vtotal;
  pv_vsyncpw_t vsyncpw;
  u_short vgstart;
  u_short vgwidth;
  u_short hoffset;
  u_short hactive;
  u_short voffset;
  u_short vactive;
  pv_strbdly_t strbdly;
  pv_frstdly_t frstdly;
  pv_con1_t con1;
  pv_con2_t con2;
  pv_trigcon_t trigcon;
  u_short sw_trig;
  pv_p_clamp_t p_clamp;
  pv_pll_reg_t pllprg;
  u_short reserved_4;
  u_short reserved_5;
  u_short reserved_6;
  u_short reserved_7;
/* here is the end of the real regiter space
   since we use this structure for both (register access
   and camera configuration storage, we append some
   definitions for registers, that are access indirectly */
  u_short fdiv;
  u_short rdiv;
  u_short vcogain;
  u_short pda;
  u_short lcnt;
  u_short nref;
  u_short pref;
};

#define pv_cam_t pv_reg_t

/* to handle different modules in the same way */
union icp_dev_t
{
  struct vs_cam_t vs_cam;
  struct dg_cam_t dg_cam;
  struct pv_cam_t pv_cam;
};

struct icp_acc_t
{
  /* grab statistics */
  u_long frames_captured;	/* overall number of frames */
  u_long frames_valid;	/* task attached and saved */
  u_long frames_lost;	/* task attached and lost */
  /* ioport statistics */
  u_long events_captured;	/* overall number of captured interrupts */
  u_long bytes_valid;	/* inport data are stored (read(2) assigned) */
  u_long bytes_lost;	/* ioport data are lost (no process in read) */
};

/* snap statistics */

#ifndef EXT_OS_DEFINED	/* if nothing is defined, fallback */
struct timeval
{
  long	tv_sec;		/* seconds */
  long	tv_usec;	/* microseconds */
};
#endif

struct icp_info_t {
  unsigned long	clock_tics;
  struct timeval time_stamp;
  struct icp_acc_t frame_nums;
};

/* delay time */
#ifndef EXT_OS_DEFINED	/* if nothing is defined, fallback */
struct timespec
{
  long	tv_sec;		/* seconds */
  long	tv_nsec;	/* nanoseconds */
};
#endif

#ifdef linux

/* IC-PCI ioctl defines use 'G' for coding and range from 0 to 31 */

#define GIOC_CR_SET_FREEZE	_IO  ('G', 0)
#define GIOC_CR_SET_GRAB	_IO  ('G', 1)
#define GIOC_CR_SET_SNAP	_IOW ('G', 2, int)
#define GIOC_CR_SET_NORMAL	_IO  ('G', 3)
#define GIOC_CR_SET_TRIGGER	_IO  ('G', 4)
#define GIOC_CR_SET_SOFTTRIG	_IO  ('G', 5)
#define GIOC_CR_SET_HVDEC	_IOW ('G', 6, int)

/* AM-VS ioctl defines use 'G' for coding and range from 96 to 127 */

#define GIOC_VS_GET_LUT		_IOR ('G',  96, struct vs_lut_t)
#define GIOC_VS_SET_LUT		_IOW ('G',  97, struct vs_lut_t)
#define GIOC_VS_SET_LUT_LIN	_IO  ('G',  98)
#define GIOC_VS_GET_NREF	_IOR ('G',  99, int)
#define GIOC_VS_SET_NREF	_IOW ('G', 100, int)
#define GIOC_VS_GET_PREF	_IOR ('G', 101, int)
#define GIOC_VS_SET_PREF	_IOW ('G', 102, int)

/* AM-DIG ioctl defines use 'G' for coding and range from 128 to 159 */

#define GIOC_DG_GET_LUTPG	_IOR ('G', 128, int)
#define GIOC_DG_SET_LUTPG	_IOW ('G', 129, int)
#define GIOC_DG_GET_LUT		_IOR ('G', 130, union dg_lut16_t)
#define GIOC_DG_SET_LUT		_IOW ('G', 131, union dg_lut16_t)
#define GIOC_DG_GET_OLUTPG	_IOR ('G', 132, int)
#define GIOC_DG_SET_OLUTPG	_IOW ('G', 133, int)
#define GIOC_DG_GET_OLUT	_IOR ('G', 134, struct dg_lut8_t)
#define GIOC_DG_SET_OLUT	_IOW ('G', 135, struct dg_lut8_t)
#define GIOC_DG_SET_LUT_LIN	_IO  ('G', 136)
#define GIOC_DG_SET_NORM_TO8	_IO  ('G', 137)
#define GIOC_DG_SET_NORM_TO16	_IO  ('G', 138)
#define GIOC_DG_GET_EXTIME	_IOR ('G', 139, struct timespec)
#define GIOC_DG_SET_EXTIME	_IOW ('G', 140, struct timespec)

/* GENERIC ioctl defines use 'G' for coding and range from 224 to 255 */

#define	GIOC_GE_GET_VERSION	_IOR ('G', 224, int)
#define GIOC_GE_GET_MODULE	_IOR ('G', 225, int)

#define GIOC_GE_GET_BMAMAPSIZE	_IOR ('G', 226, size_t)
#define GIOC_GE_GET_BMABUFFERS	_IOR ('G', 227, int)
#define GIOC_GE_GET_BMAWIDTH	_IOR ('G', 228, int)
#define GIOC_GE_GET_BMAHEIGHT	_IOR ('G', 229, int)
#define GIOC_GE_GET_BMABUFSIZE	_IOR ('G', 230, size_t)
#define GIOC_GE_GET_BMABUF	_IOR ('G', 231, int)
#define GIOC_GE_SET_BMABUF	_IOW ('G', 232, int)
#define GIOC_GE_GET_BMABUFADDR	_IOR ('G', 233, off_t)

#define GIOC_GE_GET_MEMMAPSIZE	_IOR ('G', 234, size_t)
#define GIOC_GE_GET_MEMBUFFERS	_IOR ('G', 235, int)
#define GIOC_GE_GET_MEMWIDTH	_IOR ('G', 236, int)
#define GIOC_GE_GET_MEMHEIGHT	_IOR ('G', 237, int)
#define GIOC_GE_GET_MEMBUFSIZE	_IOR ('G', 238, size_t)
#define GIOC_GE_GET_MEMBUF	_IOR ('G', 239, int)
#define GIOC_GE_SET_MEMBUF	_IOW ('G', 240, int)
#define GIOC_GE_GET_MEMBUFADDR	_IOR ('G', 241, off_t)

#define GIOC_GE_GET_CAMERA	_IOR ('G', 242, int)
#define GIOC_GE_SET_CAMERA	_IOW ('G', 243, int)
#define GIOC_GE_GET_CAMCNF	_IOR ('G', 244, union icp_dev_t)
#define GIOC_GE_SET_CAMCNF	_IOW ('G', 245, union icp_dev_t)

#define GIOC_GE_GET_FPS		_IOR ('G', 246, float)
#define GIOC_GE_SET_FPS		_IOW ('G', 247, float)

#define GIOC_GE_GET_STATS	_IOR ('G', 248, struct icp_acc_t)
#define GIOC_GE_SET_STATS	_IO  ('G', 249)

/* PCVision ioctl defines use 'H' for coding and range from 96 to 127 */

#define HIOC_PV_GET_LUT		_IOR ('H',  96, struct vs_lut_t)
#define HIOC_PV_SET_LUT		_IOW ('H',  97, struct vs_lut_t)
#define HIOC_PV_SET_LUT_LIN	_IO  ('H',  98)
#define HIOC_PV_GET_NREF	_IOR ('H',  99, int)
#define HIOC_PV_SET_NREF	_IOW ('H', 100, int)
#define HIOC_PV_GET_PREF	_IOR ('H', 101, int)
#define HIOC_PV_SET_PREF	_IOW ('H', 102, int)

#endif /* linux */

#ifndef EXT_OS_DEFINED	/* if nothing is defined, fallback */
#define GIOC_CR_SET_FREEZE	1
#define GIOC_CR_SET_GRAB	2
#define GIOC_CR_SET_SNAP	3
#define GIOC_CR_SET_NORMAL	4
#define GIOC_CR_SET_TRIGGER	5
#define GIOC_CR_SET_SOFTTRIG	6
#define GIOC_CR_SET_HVDEC	7 
#define GIOC_VS_GET_LUT		8 
#define GIOC_VS_SET_LUT		9 
#define GIOC_VS_SET_LUT_LIN	10
#define GIOC_VS_GET_NREF	11
#define GIOC_VS_SET_NREF	12
#define GIOC_VS_GET_PREF	13
#define GIOC_VS_SET_PREF	14
#define GIOC_DG_GET_LUTPG	15
#define GIOC_DG_SET_LUTPG	16
#define GIOC_DG_GET_LUT		17
#define GIOC_DG_SET_LUT		18
#define GIOC_DG_GET_OLUTPG	19
#define GIOC_DG_SET_OLUTPG	20
#define GIOC_DG_GET_OLUT	21
#define GIOC_DG_SET_OLUT	22
#define GIOC_DG_SET_LUT_LIN	23
#define GIOC_DG_SET_NORM_TO8	24
#define GIOC_DG_SET_NORM_TO16	25
#define GIOC_DG_GET_EXTIME	26
#define GIOC_DG_SET_EXTIME	27
#define	GIOC_GE_GET_VERSION	28
#define GIOC_GE_GET_MODULE	29
#define GIOC_GE_GET_BMAMAPSIZE	30
#define GIOC_GE_GET_BMABUFFERS	31
#define GIOC_GE_GET_BMAWIDTH	32
#define GIOC_GE_GET_BMAHEIGHT	33
#define GIOC_GE_GET_BMABUFSIZE	34
#define GIOC_GE_GET_BMABUF	35
#define GIOC_GE_SET_BMABUF	36
#define GIOC_GE_GET_BMABUFADDR	37
#define GIOC_GE_GET_MEMMAPSIZE	38
#define GIOC_GE_GET_MEMBUFFERS	39
#define GIOC_GE_GET_BMAWIDTH	40
#define GIOC_GE_GET_BMAHEIGHT	41
#define GIOC_GE_GET_MEMBUFSIZE	42
#define GIOC_GE_GET_MEMBUF	43
#define GIOC_GE_SET_MEMBUF	44
#define GIOC_GE_GET_MEMBUFADDR	45
#define GIOC_GE_GET_CAMERA	46
#define GIOC_GE_SET_CAMERA	47
#define GIOC_GE_GET_CAMCNF	48
#define GIOC_GE_SET_CAMCNF	49
#define GIOC_GE_GET_FPS		50
#define GIOC_GE_SET_FPS		51
#define GIOC_GE_GET_STATS	52
#define GIOC_GE_SET_STATS	53
#define HIOC_PV_GET_LUT		54
#define HIOC_PV_SET_LUT		55
#define HIOC_PV_SET_LUT_LIN	56
#define HIOC_PV_GET_NREF	57
#define HIOC_PV_SET_NREF	58
#define HIOC_PV_GET_PREF	59
#define HIOC_PV_SET_PREF	60
#endif /* nothing defined */

#endif /* ICPCIEXT_H */

